package cn.ibizlab.core.extensions.service;

import cn.ibizlab.core.extensions.domain.EngineMQMsg;
import cn.ibizlab.core.extensions.util.MQConsumeMsgListenerProcessor;
import cn.ibizlab.core.lite.extensions.domain.EntityModel;
import cn.ibizlab.core.lite.extensions.domain.EntityObj;
import cn.ibizlab.core.lite.extensions.service.LiteDataCallback;
import cn.ibizlab.core.lite.extensions.service.LiteDataService;
import cn.ibizlab.core.lite.extensions.service.LiteModelService;
import cn.ibizlab.core.lite.mapper.TableSyncMapper;
import cn.ibizlab.core.lite.service.impl.TableSyncServiceImpl;
import cn.ibizlab.util.filter.QueryFilter;
import cn.ibizlab.util.helper.DataObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import cn.ibizlab.core.lite.domain.TableSync;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Primary;
import org.springframework.util.StringUtils;

import java.sql.Timestamp;
import java.sql.Wrapper;
import java.util.*;

/**
 * 实体[ETL表同步] 自定义服务对象
 */
@Slf4j
@Primary
@Service("TableSyncExService")
public class TableSyncExService extends TableSyncServiceImpl {

    @Override
    protected Class currentModelClass() {
        return com.baomidou.mybatisplus.core.toolkit.ReflectionKit.getSuperClassGenericType(this.getClass().getSuperclass(), 1);
    }

    /**
     * [Sync:Sync] 行为扩展
     * @param et
     * @return
     */
    @Override
    public TableSync sync(TableSync et) {
        List<TableSync> list = null;
        String params= DataObject.getStringValue(et.get("param"),"");
        LambdaQueryWrapper<TableSync> wrapper= Wrappers.<TableSync>lambdaQuery();
        if(!StringUtils.isEmpty(params))
            wrapper.like(TableSync::getGroup,params);
        if(!StringUtils.isEmpty(et.getId()))
            wrapper.eq(TableSync::getId,et.getId());
        wrapper.eq(TableSync::getIsEnable,1);
        list=this.list(wrapper);
        sync(list);
        return super.sync(et);
    }

    @Autowired
    private LiteModelService liteModelService;


    @Autowired
    private LiteDataService liteDataService;

    public void sync(List<TableSync> list)
    {
        for(TableSync sync:list)
        {
            try
            {
                if(sync.getIsEnable()!=null&&sync.getIsEnable()==0)
                    continue;
                String sourceSystemId=sync.getSourceSystemId();
                String sourceDsId=sync.getSourceDsId();
                String sourceEntityName=sync.getSourceEntityName();
                String systemId=sync.getSystemId();
                String dsId=sync.getDsId();
                String entityName=sync.getEntityName();
                if(StringUtils.isEmpty(entityName)||StringUtils.isEmpty(systemId))
                    continue;
                if(StringUtils.isEmpty(sourceEntityName))
                    sourceEntityName=entityName;
                if(StringUtils.isEmpty(sourceSystemId))
                    sourceSystemId=systemId;
                EntityModel entityModel=liteModelService.getEntityModel(systemId,entityName);
                if(StringUtils.isEmpty(dsId))
                    dsId=entityModel.getDsName();
                EntityModel sourceModel=liteModelService.getEntityModel(sourceSystemId,sourceEntityName);
                if(StringUtils.isEmpty(sourceDsId))
                    sourceDsId=sourceModel.getDsName();
                if(sourceDsId.equals(dsId)&&sourceEntityName.equals(entityName))
                    continue;


                Timestamp lastTimestamp=DataObject.getTimestampValue(sync.getEtlTimestamp(),DataObject.getBeginDate());


                {
                    Calendar c = Calendar.getInstance();
                    c.setTime(lastTimestamp);
                    c.add(Calendar.HOUR, -8);
                    lastTimestamp=new java.sql.Timestamp(c.getTime().getTime());
                }




                QueryFilter filter=QueryFilter.createQuery();
                if(sourceModel.getLastModifyField()!=null)
                    filter.ge(sourceModel.getLastModifyField().getColumnName(),lastTimestamp);
                Timestamp runTimestamp=new Timestamp(System.currentTimeMillis());


                final String dsName=dsId;
                liteDataService.searchCursor(sourceModel,sync.getSourceExpression(),sourceDsId,filter, new LiteDataCallback<List<EntityObj>>() {
                    @Override
                    public void total(Integer total) {

                        if(total > 0)
                        {
                            String updateSql = "UPDATE ibztablesync SET lastread=#{et.lastread},lastwrite=0," +
                                    "lastruntime=#{et.lastruntime}," +
                                    "lastrunresult='RUNNING' WHERE syncid = #{et.syncid}";
                            HashMap<String, Object> param = new HashMap<>();
                            param.put("lastread", total);
                            param.put("lastruntime", runTimestamp);
                            param.put("syncid", sync.getId());
                            updateBySQL(updateSql,param);
                        }
                        else {
                            String updateSql = "UPDATE ibztablesync SET lastread=0,lastwrite=0," +
                                    "lastruntime=#{et.lastruntime}," +
                                    "lastendtime=#{et.lastendtime}," +
                                    "etltimestamp=#{et.etltimestamp}," +
                                    "lastrunresult='FINISH' WHERE syncid = #{et.syncid}";
                            HashMap<String, Object> param = new HashMap<>();
                            param.put("lastendtime", new Timestamp(System.currentTimeMillis()));
                            param.put("lastruntime", runTimestamp);
                            param.put("etltimestamp", runTimestamp);
                            param.put("syncid", sync.getId());
                            updateBySQL(updateSql,param);

                        }
                    }

                    @Override
                    public boolean processData(List<EntityObj> datas) {
                        boolean rt=liteDataService.saveBatch(entityModel,dsName,datas);

                        if(rt)
                        {
                            String updateSql = "UPDATE ibztablesync SET lastwrite=(CASE WHEN lastwrite is null then 0 else lastwrite end+#{et.count}) WHERE syncid = #{et.syncid}";
                            HashMap<String, Object> param = new HashMap<>();
                            param.put("count", datas.size());
                            param.put("syncid", sync.getId());
                            updateBySQL(updateSql,param);

                            updateSql = "UPDATE ibztablesync SET "+
                                    "lastendtime=#{et.lastendtime}," +
                                    "etltimestamp=#{et.etltimestamp}," +
                                    "lastrunresult='FINISH' WHERE syncid = #{et.syncid} and lastread <= lastwrite ";
                            param = new HashMap<>();
                            param.put("lastendtime", new Timestamp(System.currentTimeMillis()));
                            param.put("etltimestamp", runTimestamp);
                            param.put("syncid", sync.getId());
                            updateBySQL(updateSql,param);
                        }
                        else
                        {
                            String updateSql = "UPDATE ibztablesync SET lastwrite=CASE WHEN lastwrite is null then 0 else lastwrite end," +
                                    "lastruntime=#{et.lastruntime}," +
                                    "lastendtime=#{et.lastendtime}," +
                                    //"etltimestamp=#{et.etltimestamp}," +
                                    "lastrunresult='ERROR' WHERE syncid = #{et.syncid}";
                            HashMap<String, Object> param = new HashMap<>();
                            param.put("lastendtime", new Timestamp(System.currentTimeMillis()));
                            param.put("lastruntime", runTimestamp);
                            //param.put("etltimestamp", runTimestamp);
                            param.put("syncid", sync.getId());
                            updateBySQL(updateSql,param);
                        }


                        return rt;
                    }
                });
            }
            catch (Exception ex)
            {
                log.error("sync同步失败:"+sync.getEntityName()+":"+ex.getMessage());
                ex.printStackTrace();

            }
        }

    }

    private void updateBySQL(String updateSql , Map param)
    {
        try {
            DynamicDataSourceContextHolder.push("master");
            this.getBaseMapper().updateBySQL(updateSql, param);
        } catch (Exception ex) {
            log.error("详细错误信息：" + ex.getMessage() );
        } finally {
            DynamicDataSourceContextHolder.poll();
        }
    }

}

