package cn.ibizlab.core.extensions.util;

import cn.ibizlab.core.extensions.domain.EngineMQMsg;
import cn.ibizlab.core.extensions.domain.ResultsMQMsg;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.exception.MQBrokerException;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.remoting.exception.RemotingException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

/**
 * MQ默认发送消息服务
 */
@Slf4j
@Service
@ConditionalOnExpression("(${rocketmq.producer.enabled:false})||(${rocketmq.consumer.enabled:false})")
public class DefaultMQProducerService implements MsgProducerService{

    @Value("${rocketmq.producer.ruleEngineTopic:DSTMSG}")
    private String ruleEngineTopic;

    @Value("${rocketmq.producer.resultsTopic:DSTRESULTSMSG}")
    private String resultsTopic;

    @Autowired
    @Lazy
    DefaultMQProducer defaultMQProducer;



    @Override
    public void sendEngineMsg(EngineMQMsg engineMQMsg) throws InterruptedException, RemotingException, MQClientException, MQBrokerException {
        String msg = JSON.toJSONString(engineMQMsg);
        Message sendMsg = new Message(ruleEngineTopic, "Engine", msg.getBytes());
        SendResult sendResult = defaultMQProducer.send(sendMsg);
        log.info("引擎消息发送响应：" + sendResult.toString());
    }

    @Override
    public void sendBuildMsg(EngineMQMsg engineMQMsg) throws InterruptedException, RemotingException, MQClientException, MQBrokerException  {
        String msg = JSON.toJSONString(engineMQMsg);
        Message sendMsg = new Message(ruleEngineTopic, "Build", msg.getBytes());
        SendResult sendResult = defaultMQProducer.send(sendMsg);
        log.info("引擎消息发送响应：" + sendResult.toString());
    }

    @Override
    public void sendResultsMsg(ResultsMQMsg resultsMQMsg) throws InterruptedException, RemotingException, MQClientException, MQBrokerException  {
        String msg = JSON.toJSONString(resultsMQMsg);
        Message sendMsg = new Message(resultsTopic, "default", msg.getBytes());
        sendMsg.setKeys(resultsMQMsg.getKeyValueField());
        SendResult sendResult = defaultMQProducer.send(sendMsg);
        log.info("构建消息发送响应：" + sendResult.toString());
    }
}
